@@ -71,6 +71,16 @@ class TestScreen(BaseTest):
         self.ae(s.cursor.x, 5), self.ae(s.cursor.y, 2)
         s.draw('c' * 15)
         self.ae(str(s.line(0)), 'ニチ')
+        s.reset()
+        qt = 'a' * s.columns + '\u0306'
+        s.draw(qt)
+        self.ae(str(s.line(0)), qt)
+        s.reset()
+        s.draw(qt[:-1]), s.draw(qt[-1])
+        self.ae(str(s.line(0)), qt)
+        s.reset()
+        s.draw(qt[:-1]), s.linefeed(), s.carriage_return(), s.draw(qt[-1])
+        self.ae(str(s.line(0)), qt[:-1])
 
         # Now test without line-wrap
         s.reset(), s.reset_dirty()
@@ -84,8 +94,10 @@ class TestScreen(BaseTest):
 
         # Now test in insert mode
         s.reset(), s.reset_dirty()
+        text = '1\u03062345'
         s.set_mode(IRM)
-        s.draw('1\u03062345' * 5)
+        s.draw(text * 5)
+        self.ae(str(s.line(0)), text)
         s.cursor_back(5)
         self.ae(s.cursor.x, 0), self.ae(s.cursor.y, 4)
         s.reset_dirty()
@@ -93,6 +105,14 @@ class TestScreen(BaseTest):
         self.ae(str(s.line(4)), 'a\u0306b1\u030623')
         self.ae((s.cursor.x, s.cursor.y), (2, 4))
 
+        # Test drawing of tabs
+        s = self.create_screen(cols=32)
+        txt = 'a\tb'
+        s.draw(txt)
+        ln = s.line(0)
+        self.ae(txt, ln.as_ansi())
+
+
     def test_rep(self):
         s = self.create_screen()
         s.draw('a')
@@ -127,9 +147,9 @@ class TestScreen(BaseTest):
         q = '\U0001f468\u200d\U0001f469\u200d\U0001f467\u200d\U0001f466'
         s.draw(q)
         self.ae(q, str(s.line(0)))
-        self.ae(s.cursor.x, 8)
+        self.ae(s.cursor.x, 2)
         for x in '\u200b\u200c\u200d':
-            s = self.create_screen()
+            s.reset()
             q = f'X{x}Y'
             s.draw(q)
             self.ae(q, str(s.line(0)))
@@ -387,6 +407,15 @@ class TestScreen(BaseTest):
         parse_bytes(s, b'\x1b[?2048h')  # ]
         self.ae(c.num_of_resize_events, 2)
 
+    def test_da1(self):
+        s = self.create_screen()
+        parse_bytes(s, b'\x1b[c\x1b[0c')  # ]]
+        self.ae(s.callbacks.da1, ['?62;52;c', '?62;52;c'])  # ]]
+        s.callbacks.clear()
+        self.create_screen(options={'clipboard_control': 'read-clipboard'})
+        parse_bytes(s, b'\x1b[c')  # ]]
+        self.ae(s.callbacks.da1, ['?62;c'])  # ]]
+
     def test_cursor_after_resize(self):
 
         def draw(text, end_line=True):
@@ -665,6 +694,12 @@ class TestScreen(BaseTest):
         self.ae(s.text_for_selection(True), ('a\x1b[32mb\x1b[39mc  ', 'xy', '\x1b[m'))
         self.ae(s.text_for_selection(True, True), ('a\x1b[32mb\x1b[39mc', 'xy', '\x1b[m'))
         # ]]]]]]]]]]]]]]]]]]]]
+        s.reset()
+        s.draw('a'), s.carriage_return(), s.linefeed(), s.linefeed(), s.draw('b')
+        s.start_selection(0, 0)
+        s.update_selection(4, 4)
+        self.ae(''.join(s.text_for_selection()), 'a\n\nb')
+        self.ae(''.join(s.text_for_selection(True)), 'a\n\nb')
 
     def test_soft_hyphen(self):
         s = self.create_screen()
@@ -675,7 +710,24 @@ class TestScreen(BaseTest):
         self.ae(s.text_for_selection(), ('a\u00adb',))
 
     def test_variation_selectors(self):
+        s = self.create_screen(cols=8)
+        def widths(text, *widths):
+            s.reset()
+            s.draw(text)
+            def w(x):
+                c = s.cpu_cells(0, x)
+                return (c['mcd'] or {'width': 1})['width']
+            actual = tuple(w(x) for x in range(len(widths)))
+            self.ae(widths, actual)
+        widths('\u4e00\u4e00\u26ab\ufe0e', 2, 2, 2, 2, 1)
+
         s = self.create_screen()
+        def tt(text_to_draw):
+            s.reset()
+            s.draw(text_to_draw)
+            self.ae(str(s.line(0)), text_to_draw)
+        tt('abc\U0001f44d\ufe0ed')
+
         def t(*a):
             s.reset()
             for i in range(0, len(a), 2):
@@ -907,6 +959,13 @@ class TestScreen(BaseTest):
         def set_link(url=None, id=None):
             parse_bytes(s, '\x1b]8;id={};{}\x1b\\'.format(id or '', url or '').encode('utf-8'))
 
+        set_link('wide-chars', 'XX')
+        self.ae(s.line(0).hyperlink_ids(), tuple(0 for x in range(s.columns)))
+        s.draw('状')
+        self.ae(s.line(0).hyperlink_ids(), (1, 1) + tuple(0 for x in range(s.columns - 2)))
+        set_link()
+
+        s = self.create_screen()
         set_link('url-a', 'a')
         self.ae(s.line(0).hyperlink_ids(), tuple(0 for x in range(s.columns)))
         s.draw('a')
