@@ -11,8 +11,11 @@ import (
 	"runtime"
 	"slices"
 	"strconv"
+	"strings"
+	"sync"
 
 	"golang.org/x/exp/constraints"
+	"golang.org/x/text/language"
 )
 
 var _ = fmt.Print
@@ -64,6 +67,72 @@ func Filter[T any](s []T, f func(x T) bool) []T {
 	return ans
 }
 
+func Format_stacktrace_on_panic(r any) (text string, err error) {
+	pcs := make([]uintptr, 512)
+	n := runtime.Callers(3, pcs)
+	lines := []string{}
+	frames := runtime.CallersFrames(pcs[:n])
+	err = fmt.Errorf("Panicked: %s", r)
+	lines = append(lines, fmt.Sprintf("\r\nPanicked with error: %s\r\nStacktrace (most recent call first):\r\n", r))
+	found_first_frame := false
+	for frame, more := frames.Next(); more; frame, more = frames.Next() {
+		if !found_first_frame {
+			if strings.HasPrefix(frame.Function, "runtime.") {
+				continue
+			}
+			found_first_frame = true
+		}
+		lines = append(lines, fmt.Sprintf("%s\r\n\t%s:%d\r\n", frame.Function, frame.File, frame.Line))
+	}
+	text = strings.Join(lines, "")
+	return strings.TrimSpace(text), err
+}
+
+// Run the specified function in parallel over chunks from the specified range.
+// If the function panics, it is turned into a regular error.
+func Run_in_parallel_over_range(num_procs int, f func(int, int) error, start, limit int) (err error) {
+	num_items := limit - start
+	if num_procs <= 0 {
+		num_procs = runtime.NumCPU()
+	}
+	num_procs = max(1, min(num_procs, num_items))
+	if num_procs < 2 {
+		defer func() {
+			if r := recover(); r != nil {
+				stacktrace, e := Format_stacktrace_on_panic(r)
+				err = fmt.Errorf("%s\n\n%w", stacktrace, e)
+			}
+		}()
+		return f(start, limit)
+	}
+	chunk_sz := max(1, num_items/num_procs)
+	var wg sync.WaitGroup
+	echan := make(chan error, num_procs)
+	for start < limit {
+		end := min(start+chunk_sz, limit)
+		wg.Add(1)
+		go func(start, end int) {
+			defer func() {
+				if r := recover(); r != nil {
+					stacktrace, e := Format_stacktrace_on_panic(r)
+					echan <- fmt.Errorf("%s\n\n%w", stacktrace, e)
+				}
+				wg.Done()
+			}()
+			if err := f(start, end); err != nil {
+				echan <- err
+			}
+		}(start, end)
+		start = end
+	}
+	wg.Wait()
+	close(echan)
+	for qerr := range echan {
+		return qerr
+	}
+	return
+
+}
 func Map[T any, O any](f func(x T) O, s []T) []O {
 	ans := make([]O, 0, len(s))
 	for _, x := range s {
@@ -74,7 +143,7 @@ func Map[T any, O any](f func(x T) O, s []T) []O {
 
 func Repeat[T any](x T, n int) []T {
 	ans := make([]T, n)
-	for i := 0; i < n; i++ {
+	for i := range n {
 		ans[i] = x
 	}
 	return ans
@@ -90,6 +159,18 @@ func StableSort[T any](s []T, cmp func(a, b T) int) []T {
 	return s
 }
 
+func Uniq[T comparable](s []T) []T {
+	seen := NewSet[T](len(s))
+	ans := make([]T, 0, len(s))
+	for _, x := range s {
+		if !seen.Has(x) {
+			seen.Add(x)
+			ans = append(ans, x)
+		}
+	}
+	return ans
+}
+
 func sort_with_key[T any, C constraints.Ordered](stable bool, s []T, key func(a T) C) []T {
 	type t struct {
 		key C
@@ -317,3 +398,28 @@ func Abs[T constraints.Integer](x T) T {
 	}
 	return x
 }
+
+var LanguageTag = sync.OnceValue(func() language.Tag {
+	// Check environment variables in order of precedence
+	var locale string
+	for _, v := range []string{"LC_ALL", "LC_MESSAGES", "LANG"} {
+		locale = os.Getenv(v)
+		if locale != "" {
+			break
+		}
+	}
+	if locale == "" {
+		return language.English // Default/fallback
+	}
+	// Remove encoding, e.g., ".UTF-8"
+	locale = strings.Split(locale, ".")[0]
+	// Replace underscore with hyphen to match BCP47 format (en_US -> en-US)
+	locale = strings.ReplaceAll(locale, "_", "-")
+	// Validate/normalize with golang.org/x/text/language
+	tag, err := language.Parse(locale)
+	if err != nil {
+		return language.English
+	}
+	return tag
+
+})
